<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2023 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Core\Html;

use _JchOptimizeVendor\Psr\Http\Message\UriInterface;
use JchOptimize\Core\Uri\Utils;

class AttributesCollection
{
    /**
     * @var array<string, Attribute>
     */
    protected array $attributes = [];
    protected bool $isXhtml;
    protected array $booleanAttributes = ['allowfullscreen', 'async', 'autofocus', 'autoplay', 'checked', 'controls', 'default', 'defer', 'disabled', 'formnovalidate', 'inert', 'ismap', 'itemscope', 'loop', 'multiple', 'muted', 'nomodule', 'novalidate', 'open', 'playsinline', 'readonly', 'required', 'reversed', 'selected'];
    protected array $enumeratedEmptyStringValue = ['crossorigin' => 'anonymous', 'preload' => 'auto', 'lazy' => 'eager', 'fetchpriority' => 'auto', 'autocomplete' => 'on', 'hidden' => 'hidden', 'contenteditable' => 'true'];

    public function __construct(bool $isXhtml)
    {
        $this->isXhtml = $isXhtml;
    }

    public function setAttribute(string $name, $value = null, string $delimiter = null): void
    {
        if ('src' == $name || 'href' == $name || 'poster' == $name) {
            $value = $this->urlValue($value);
        }
        if ('class' == $name) {
            $value = $this->classValue($value);
        }
        if ($this->has($name)) {
            $attribute = $this->attributes[$name];
        } else {
            $isBoolean = $this->isBoolean($name);
            $attribute = (new \JchOptimize\Core\Html\Attribute())->name($name)->boolean($isBoolean);
        }
        if (null !== $value) {
            $attribute->value($value);
        }
        if (null !== $delimiter) {
            $attribute->delimiter($delimiter);
        }
        $this->attributes[$name] = $attribute;
    }

    /**
     * @return array|Attribute|string|true|UriInterface
     */
    public function getValue(string $name)
    {
        return $this->attributes[$name]->value();
    }

    public function setAttributes(array $attributes): void
    {
        foreach ($attributes as $name => $value) {
            $this->setAttribute($name, $value);
        }
    }

    public function removeAttribute(string $name): void
    {
        if (isset($this->attributes[$name])) {
            unset($this->attributes[$name]);
        }
    }

    public function render(): string
    {
        $attributes = '';
        foreach ($this->attributes as $attribute) {
            $name = $attribute->name();
            $value = $attribute->value();
            $delimiter = $attribute->delimiter();
            if ($this->isXhtml) {
                if ($attribute->boolean()) {
                    $value = \preg_replace('#^data-#', '', $name);
                }
                if (\array_key_exists($name, $this->enumeratedEmptyStringValue) && empty($value)) {
                    $value = $this->enumeratedEmptyStringValue[$name];
                }
                if ('' == $delimiter) {
                    $delimiter = '"';
                }
            }
            if (\is_array($value)) {
                if (\count($value) > 1 && '' == $delimiter) {
                    $delimiter = '"';
                }
                $value = \implode(' ', $value);
            }
            if ($value instanceof UriInterface) {
                $value = (string) $value;
            }
            if (($attribute->boolean() || \array_key_exists($name, $this->enumeratedEmptyStringValue) && empty($value)) && !$this->isXhtml) {
                $attributes .= " {$name}";
            } else {
                $attributes .= " {$name}={$delimiter}{$value}{$delimiter}";
            }
        }

        return $attributes;
    }

    public function isBoolean($name): bool
    {
        return \in_array(\preg_replace('#^data-#', '', $name), $this->booleanAttributes);
    }

    public function has($name): bool
    {
        return \array_key_exists($name, $this->attributes);
    }

    private function urlValue($value): UriInterface
    {
        return Utils::uriFor($value);
    }

    private function classValue($value): array
    {
        if (\is_string($value)) {
            $value = \explode(' ', $value);
        }
        if (isset($this->attributes['class'])) {
            $value = \array_unique(\array_merge($this->attributes['class']->value(), $value));
        }

        return $value;
    }
}
